﻿Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Runtime.InteropServices
Imports System.Windows.Forms
Imports System.Drawing
Namespace HPRT.Printer
    Public Class PrinterState
        Public Shared ReadOnly STS_NORMAL As UInteger = &H0
        Public Shared ReadOnly STS_PAPEREMPTY As UInteger = 1
        Public Shared ReadOnly STS_COVEROPEN As UInteger = 2
        Public Shared ReadOnly STS_PAPERNEAREND As UInteger = 4
        Public Shared ReadOnly STS_MSR_READY As UInteger = 8
        Public Shared ReadOnly STS_SMARTCARD_READY As UInteger = 16
        Public Shared ReadOnly STS_ERROR As UInteger = 32
        Public Shared ReadOnly STS_NOT_OPEN As UInteger = 64
        Public Shared ReadOnly STS_OFFLINE As UInteger = 128
        Private m_status As UInteger = 0
        Public Sub New()
            m_status = 0
        End Sub
        Public Property State() As UInteger
            Get
                Return m_status
            End Get
            Set(ByVal value As UInteger)
                m_status = value
            End Set
        End Property
        Public ReadOnly Property IsNormal() As Boolean
            Get
                Return (STS_NORMAL = m_status)
            End Get
        End Property
        Public ReadOnly Property InError() As Boolean
            Get
                Return (PAPEREMPTY OrElse COVEROPEN OrElse PAPERNEAREND OrElse MSR_READY OrElse SMARTCARD_READY OrElse [ERROR] OrElse NOT_OPEN OrElse OFFLINE)
            End Get
        End Property
        Public ReadOnly Property PAPEREMPTY() As Boolean
            Get
                Return ((STS_PAPEREMPTY And m_status) > 0)
            End Get
        End Property
        Public ReadOnly Property COVEROPEN() As Boolean
            Get
                Return ((STS_COVEROPEN And m_status) > 0)
            End Get
        End Property
        Public ReadOnly Property PAPERNEAREND() As Boolean
            Get
                Return ((STS_PAPERNEAREND And m_status) > 0)
            End Get
        End Property
        Public ReadOnly Property MSR_READY() As Boolean
            Get
                Return ((STS_MSR_READY And m_status) > 0)
            End Get
        End Property
        Public ReadOnly Property SMARTCARD_READY() As Boolean
            Get
                Return ((STS_SMARTCARD_READY And m_status) > 0)
            End Get
        End Property
        Public ReadOnly Property [ERROR]() As Boolean
            Get
                Return ((STS_ERROR And m_status) > 0)
            End Get
        End Property
        Public ReadOnly Property NOT_OPEN() As Boolean
            Get
                Return ((STS_NOT_OPEN And m_status) > 0)
            End Get
        End Property
        Public ReadOnly Property OFFLINE() As Boolean
            Get
                Return ((STS_OFFLINE And m_status) > 0)
            End Get
        End Property
    End Class

    Public Class HPRTPrinter
        Public Const HPRTDIR As String = "TSPL_SDK.dll"
        Public Const charSet As CharSet = CharSet.Ansi
        <DllImport(HPRTDIR, charSet:=charSet, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function PrtFormatError(ByVal error_no As Integer, ByVal langid As Integer, ByVal buf As Byte(), ByVal pos As Integer, ByVal bufSize As Integer) As Integer
        End Function
        <DllImport(HPRTDIR, charSet:=charSet, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function PrtPrinterCreator(ByRef printer As IntPtr, ByVal model As String) As Integer
        End Function
        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function PrtPrinterDestroy(ByVal printer As IntPtr) As Integer
        End Function
        <DllImport(HPRTDIR, charSet:=charSet, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function PrtPortOpen(ByVal printer As IntPtr, ByVal portSetting As String) As Integer
        End Function
        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function PrtPortClose(ByVal printer As IntPtr) As Integer
        End Function
        '<DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        'Private Shared Function PrtPrinterInitialize(ByVal printer As IntPtr) As Integer
        'End Function

        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function TSPL_SelfTest(ByVal printer As IntPtr) As Integer
        End Function

        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function TSPL_BitMapA(ByVal printer As IntPtr, ByVal x_pos As Integer, ByVal y_pos As Integer, ByVal mode As Integer, ByVal filename As String) As Integer
        End Function

        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function TSPL_Setup(ByVal printer As IntPtr, ByVal label_width As Integer, ByVal label_height As Integer, ByVal speed As Integer, ByVal density As Integer, ByVal type As Integer, ByVal gap As Integer, ByVal offset As Integer) As Integer
        End Function

        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function TSPL_ClearBuffer(ByVal printer As IntPtr) As Integer
        End Function
        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function TSPL_Box(ByVal printer As IntPtr, ByVal x_start As Integer, ByVal y_start As Integer, ByVal x_end As Integer, ByVal y_end As Integer, ByVal thickness As Integer) As Integer
        End Function
        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function TSPL_BarCode(ByVal printer As IntPtr, ByVal x_pos As Integer, ByVal y_pos As Integer, ByVal code_type As Integer, ByVal height As Integer, ByVal readable As Integer, ByVal rotation As Integer, ByVal narrow As Integer, ByVal wide As Integer, ByVal code_data As String) As Integer
        End Function
        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function TSPL_QrCode(ByVal printer As IntPtr, ByVal x_pos As Integer, ByVal y_pos As Integer, ByVal ecc_level As Integer, ByVal width As Integer, ByVal mode As Integer, ByVal rotation As Integer, ByVal model As Integer, ByVal mask As Integer, ByVal data As String) As Integer
        End Function
        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function TSPL_Text(ByVal printer As IntPtr, ByVal x_pos As Integer, ByVal y_pos As Integer, ByVal font As Integer, ByVal rotation As Integer, ByVal x_multiplication As Integer, ByVal y_multiplication As Integer, ByVal content As String) As Integer
        End Function
        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function TSPL_Print(ByVal printer As IntPtr, ByVal num As Integer, ByVal copies As Integer) As Integer
        End Function
        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function TSPL_FormFeed(ByVal printer As IntPtr) As Integer
        End Function
        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function TSPL_SetTear(ByVal printer As IntPtr, ByVal on_off As Integer) As Integer
        End Function
        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function TSPL_Offset(ByVal printer As IntPtr, ByVal distance As Integer) As Integer
        End Function
        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function TSPL_Direction(ByVal printer As IntPtr, ByVal directions As Integer) As Integer
        End Function
        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function TSPL_Feed(ByVal printer As IntPtr, ByVal len As Integer) As Integer
        End Function
        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function TSPL_Home(ByVal printer As IntPtr) As Integer
        End Function
        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function TSPL_Learn(ByVal printer As IntPtr) As Integer
        End Function
        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function TSPL_GetDllVersion(ByVal printer As IntPtr) As Integer
        End Function
        <DllImport(HPRTDIR, CallingConvention:=CallingConvention.Cdecl)> _
        Private Shared Function TSPL_GetSN(ByVal printer As IntPtr, ByVal sn As String) As Integer
        End Function
        Private printer As IntPtr
        Private langid As Integer = 0
        Private E_SUCCESS As Integer = Constants.E_SUCCESS
        Private E_INVALID_PARAMETER As Integer = Constants.E_INVALID_PARAMETER
        Private state As New PrinterState()
        Private modelname As String = ""
        Public Sub New(ByVal model As String)
            Dim result As Integer = E_SUCCESS
            printer = IntPtr.Zero
            result = PrtPrinterCreator(printer, model)  '创建构造函数
            If E_SUCCESS <> result Then
                Dim errorMsg As String = FormatError(result)
                printer = IntPtr.Zero
                Throw New Exception(errorMsg)
            End If
            modelname = model
        End Sub
        Public Function FormatError(ByVal error_no As Integer) As String
            Dim temp As Byte() = New Byte(512) {}
            PrtFormatError(error_no, langid, temp, 0, 512)
            Return System.Text.Encoding.[Default].GetString(temp, 0, 512)
        End Function
        Public Property Model() As String
            Get
                Return modelname
            End Get
            Set(ByVal value As String)
                If value.ToUpper().Trim() <> modelname.ToUpper().Trim() Then
                    Dim p As IntPtr = IntPtr.Zero
                    If Constants.E_SUCCESS = PrtPrinterCreator(p, value) Then
                        PrtPrinterDestroy(printer)
                        printer = p
                        modelname = value
                    End If
                End If
            End Set
        End Property
        Public Function ChangeModel(ByVal modelName As String) As Integer
            Dim result As Integer = Constants.E_SUCCESS
            Return result
        End Function
        Public Function PortOpen(ByVal uri As String) As Integer
            If printer <> IntPtr.Zero Then
                Return PrtPortOpen(printer, uri)
            Else
                Return Constants.E_BAD_HANDLE
            End If
        End Function
        Public Function PortClose() As Integer
            Dim result As Integer = E_SUCCESS
            result = PrtPortClose(printer)
            Return result
        End Function
        'Public Function Initialize() As Integer
        '    Return PrtPrinterInitialize(printer)
        'End Function
        Public Function SelfTest() As Integer
            Return TSPL_SelfTest(printer)
        End Function
        Public Function BitMapA(ByVal x_pos As Integer, ByVal y_pos As Integer, ByVal mode As Integer, ByVal filename As String) As Integer
            Return TSPL_BitMapA(printer, x_pos, y_pos, mode, filename)
        End Function
        Public Function Setup(ByVal label_width As Integer, ByVal label_height As Integer, ByVal speed As Integer, ByVal density As Integer, ByVal type As Integer, ByVal gap As Integer, ByVal offset As Integer) As Integer
            Return TSPL_Setup(printer, label_width, label_height, speed, density, type, gap, offset)
        End Function
        Public Function ClearBuffer() As Integer
            Return TSPL_ClearBuffer(printer)
        End Function
        Public Function Box(ByVal x_start As Integer, ByVal y_start As Integer, ByVal x_end As Integer, ByVal y_end As Integer, ByVal thickness As Integer) As Integer
            Return TSPL_Box(printer, x_start, y_start, x_end, y_end, thickness)
        End Function
        Public Function BarCode(ByVal x_pos As Integer, ByVal y_pos As Integer, ByVal code_type As Integer, ByVal height As Integer, ByVal readable As Integer, ByVal rotation As Integer, ByVal narrow As Integer, ByVal wide As Integer, ByVal code_data As String) As Integer
            Return TSPL_BarCode(printer, x_pos, y_pos, code_type, height, readable, rotation, narrow, wide, code_data)
        End Function
        Public Function QrCode(ByVal x_pos As Integer, ByVal y_pos As Integer, ByVal ecc_level As Integer, ByVal width As Integer, ByVal mode As Integer, ByVal rotation As Integer, ByVal model As Integer, ByVal mask As Integer, ByVal data As String) As Integer
            Return TSPL_QrCode(printer, x_pos, y_pos, ecc_level, width, mode, rotation, model, mask, data)
        End Function
        Public Function Text(ByVal x_pos As Integer, ByVal y_pos As Integer, ByVal font As Integer, ByVal rotation As Integer, ByVal x_multiplication As Integer, ByVal y_multiplication As Integer, ByVal content As String) As Integer
            Return TSPL_Text(printer, x_pos, y_pos, font, rotation, x_multiplication, y_multiplication, content)
        End Function
        Public Function Print(ByVal num As Integer, ByVal copies As Integer) As Integer
            Return TSPL_Print(printer, num, copies)
        End Function
        Public Function FormFeed() As Integer
            Return TSPL_FormFeed(printer)
        End Function
        Public Function SetTear(ByVal on_off As Integer) As Integer
            Return TSPL_SetTear(printer, on_off)
        End Function
        Public Function Offset(ByVal distance As Integer) As Integer
            Return TSPL_Offset(printer, distance)
        End Function
        Public Function Direction(ByVal directions As Integer) As Integer
            Return TSPL_Direction(printer, directions)
        End Function
        Public Function Feed(ByVal len As Integer) As Integer
            Return TSPL_Feed(printer, len)
        End Function
        Public Function Home() As Integer
            Return TSPL_Home(printer)
        End Function
        Public Function Learn() As Integer
            Return TSPL_Learn(printer)
        End Function
        Public Function GetDllVersion() As Integer
            Return TSPL_GetDllVersion(printer)
        End Function
        Public Function GetSN(ByVal sn As String) As Integer
            Return TSPL_GetSN(printer, sn)
        End Function
    End Class
End Namespace